/*
 * User Mode Sandbox for Grid
 * Copyright (C) 2004 Tangui Morlier
 *    Grand-Large (INRIA Futurs)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "file_functions.h"


/**
 * Directory where it is allowed to execute an application
 */
#define NB_EXEC_ALLOWED_PATHS 1
char *(exec_allowed_path[NB_EXEC_ALLOWED_PATHS]) = {
  cwd
};

/**
 * Directory where the reads are allowed
 */
#define NB_READ_ALLOWED_PATHS 7
char *(read_allowed_path[NB_READ_ALLOWED_PATHS]) = {
  cwd, "/lib/", "/dev/null", "/etc/ld.so", "/usr/lib/", "/dev/tty", "/usr/X11R6/lib/"};

/**
 * Directory where the writes are allowed
 */
#define NB_WRITE_ALLOWED_PATHS 1
char *(write_allowed_path[NB_WRITE_ALLOWED_PATHS]) = {
  cwd
};


struct file_functions_struct {
  char read[255];
  char write[255];
};

void rw_file_decoder(struct ptrace_info_struct * pi, void * decoded) {
  struct file_functions_struct * rw = (struct file_functions_struct *) decoded;
  rw->read[0] = '\0';
  rw->write[0] = '\0';

  if((pi->regs.orig_eax != SYS_open) ||
     (pi->regs.orig_eax == SYS_open && pi->regs.ecx & O_WRONLY)
     ){
    decodeString(pi->pid, pi->regs.ebx, rw->write, MAX_PATH_SIZE);
  }else{
    decodeString(pi->pid, pi->regs.ebx, rw->read, MAX_PATH_SIZE);
  }
}

void link_file_decoder(struct ptrace_info_struct * pi, void * decoded) {
  struct file_functions_struct * rw = (struct file_functions_struct *) decoded;
  rw->read[0] = '\0';
  rw->write[0] = '\0';

  decodeString(pi->pid, pi->regs.ebx, rw->read, MAX_PATH_SIZE);
  decodeString(pi->pid, pi->regs.ecx, rw->write, MAX_PATH_SIZE);
}



void r_file_decoder(struct ptrace_info_struct * pi, void * decoded) {
  struct file_functions_struct * rw = (struct file_functions_struct *) decoded;
  rw->write[0] = '\0';

  decodeString(pi->pid, pi->regs.ebx, rw->read, MAX_PATH_SIZE);
}


void execve_decoder(struct ptrace_info_struct * pi, void * decoded) {
  char * exec;
  exec = (char*) decoded;

  decodeString(pi->pid, pi->regs.ebx, exec, MAX_PATH_SIZE);
}


int access_rewriter(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  if(pi->regs.eax != 0)
    return 1;
  if (!file_filter(pi, ptr, msg)) {
    pi->regs.eax = -1;
    if(ptrace(PTRACE_SETREGS, pi->pid, 0, &(pi->regs))<0) {
      printf("Access PTRACE SETREGS ERROR\n");
      
      return 0;
    }
  }
  return 1;
}

int file_rewriter_filter(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  if((pi->regs.orig_eax != SYS_open) ||
     (pi->regs.orig_eax == SYS_open && pi->regs.ecx & O_WRONLY)
     ){
    if(pi->in_or_out)
      return 1;
    return file_filter(pi, ptr, msg);
  }else{
    if(!pi->in_or_out)
      return 1;
    return file_rewriter(pi, ptr, msg);
  }
}

int file_rewriter(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  if(pi->regs.eax < 0)
    return 1;
  if (!file_filter(pi, ptr, msg)) {
    pi->regs.eax = -1;
    if(ptrace(PTRACE_SETREGS, pi->pid, 0, &(pi->regs))<0) {
      printf("File PTRACE SETREGS ERROR\n");
      return 0;
    }
  }
  return 1;
}

int file_logger(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  struct file_functions_struct rw;

  syscall_rules[pi->regs.orig_eax].decoder(pi, (void *)&rw);

  fprintf(stderr, "%s try to access to ", syscall_names[pi->regs.orig_eax]);
  if (strlen(rw.read)) {
    fprintf(stderr, "%s in reading ", rw.read);
  }
  if (strlen(rw.write)) {
    fprintf(stderr, "%s in writing ", rw.write);
  }

  fprintf(stderr, "mode \n");
  
}

int execve_logger(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  char exec[MAX_PATH_SIZE];
  syscall_rules[pi->regs.orig_eax].decoder(pi, (void *)&exec);
  fprintf(stderr, "[%d] execve %s 0x%x\n", pi->pid, exec, pi->regs.ecx);
  return 1;
}

int execve_filter(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  char real[PATH_MAX];
  char exec[PATH_MAX];
  char buffer[PATH_MAX];
  int i, found;

  syscall_rules[pi->regs.orig_eax].decoder(pi, (void *)&exec);

  if (which(exec, buffer))
    strncpy(exec, buffer, PATH_MAX);
  
  realpath(exec, real);

  found = 0;
  for(i = 0 ; i < NB_EXEC_ALLOWED_PATHS ; i++)
    if(strncmp(real, exec_allowed_path[i], strlen(exec_allowed_path[i])) == 0) {
      found = 1;
      break;
    }

  if(found) {
    return is_ptracable_bin(real);
  }

  snprintf(msg, MAX_MSG_SIZE, "do not have execussion rights on %s", exec);
  if (strcmp(exec, real)){
    strncat(msg, "\n\t(its real path is", MAX_MSG_SIZE);
    strncat(msg, real, MAX_MSG_SIZE);
    strncat(msg, ")", MAX_MSG_SIZE);
  }
  return 0;  
}

int file_filter(struct ptrace_info_struct * pi, char* ptr, char* msg) {
  struct file_functions_struct rw;
  char real[MAX_PATH_SIZE];
  int i, flag = 0, res = 0;
  bzero(&rw, sizeof(struct file_functions_struct));
  syscall_rules[pi->regs.orig_eax].decoder(pi, (void *)&rw);
  if(strlen(rw.read)) {
    realpath(rw.read,real);
    for(i = 0 ; i < NB_READ_ALLOWED_PATHS ; i++) {
      if(strncmp(real, read_allowed_path[i], strlen(read_allowed_path[i])) == 0) {
	res = 1;
	break;
      }
    }
  }

  if(strlen(rw.write)) {
    res = 0;
    flag = 1;
    realpath(rw.write, real);
    for(i = 0 ; i < NB_WRITE_ALLOWED_PATHS ; i++)
      if(strncmp(real, write_allowed_path[i], strlen(write_allowed_path[i])) == 0) {
	return 1;
      }
  }  

  if (!res) {
    snprintf(msg, MAX_MSG_SIZE, "do not have %s acces on %s", flag ?  "write" : "read", flag ?  rw.write :  rw.read);
    if (strcmp(flag ?  rw.write  : rw.read, real)) {
      snprintf(msg, MAX_MSG_SIZE, "%s \n\t(its real path is %s)", msg, real);
    }
  }
  return res;
}
