/*
 * User Mode Sandbox for Grid
 * Copyright (C) 2004 Tangui Morlier
 *    Grand-Large (INRIA Futurs)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "fork_functions.h"
#include <string.h>
#include <errno.h>
 
extern int errno;

//void rw_file_decoder(struct ptrace_info_struct * pi, void * decoded);
int fork_attitude(struct ptrace_info_struct * pi, char* ptr, char* msg){
  int i, j=0;
  /*  if (fork_done) {
    fork_done = 0;
    return 1;
    }*/

  i = ptrace(PTRACE_ATTACH, pi->regs.eax, 0, 0);

  fprintf(stderr, "fork: trace %d\n", pi->regs.eax);

  //  pit_t pid;
  //  if(ptrace(PTRACE_ATTACH, pi->regs.eax, 0, 0) == -1){
  if(i == -1){
    snprintf(msg, MAX_MSG_SIZE, "Can sandbox the new process (%d), %s\n", 
	     pi->regs.eax, strerror(errno));
    kill(pi->regs.eax, 9);
    return 0;
  }

  //else    fork_done = 1;

  return 1;
}

int kill_attitude(struct ptrace_info_struct * pi, char* ptr, char* msg){
  int i;

  strncpy(msg, "You can only send signals to yours processes", MAX_MSG_SIZE);
  for(i=0 ; i<MAX_PID_INFO ; i++){
    if(pid_infos[i].pi.pid == pi->regs.ebx)
      return 1;
    if(pid_infos[i].pi.pid == 0)
      return 0;
  }  
  return 0;
}


int signals_filter(struct ptrace_info_struct * pi, char* ptr, char* msg){
  if(pi->regs.ebx == SIGTRAP) {
    strncpy(msg, "Changing action on signal SIGTRAP is not allowed", MAX_MSG_SIZE);    
    return 0;
  }
  if(pi->regs.orig_eax == SYS_rt_sigprocmask || pi->regs.orig_eax == SYS_sigprocmask)
    return sigset_filter(pi, ptr, msg);

  return 1;
}


void sigset_decoder (struct ptrace_info_struct * pi, void * decoded) {
  if (decodeStruct(pi->pid, pi->regs.ebx, decoded, sizeof(sigset_t)) == -1)
    decoded = NULL;
}

void sigprocmask_decoder (struct ptrace_info_struct * pi, void * decoded) {
  if (decodeStruct(pi->pid, pi->regs.ecx, decoded, sizeof(sigset_t)) == -1)
    decoded = NULL;
}

void sigaction_decoder (struct ptrace_info_struct * pi, void * decoded) {
  struct sigaction sigaction_struct;
  if (decodeStruct(pi->pid, pi->regs.ecx, &sigaction_struct, sizeof(struct sigaction)) == -1)
    decoded = NULL;
  else
    memcpy(decoded, &sigaction_struct.sa_mask, sizeof(sigset_t));
}

int sigset_filter(struct ptrace_info_struct * pi, char* ptr, char* msg){
  long word;
  int i;
  sigset_t set;

  
  syscall_rules[pi->regs.orig_eax].decoder(pi, &set);

  if (&set == NULL) {
    snprintf(msg, MAX_MSG_SIZE, "sigset_filter error !!\n");
    return 0;
  }

  if(sigismember(&set, SIGBARRIER)) {
    snprintf(msg, MAX_MSG_SIZE, "Changing mask on signal #%d is not allowed", SIGBARRIER);    
    return 0;
  }
  return 1;

}
