/*
 * User Mode Sandbox for Grid
 * Copyright (C) 2004 Tangui Morlier
 * Copyright (C) 2004 Grand-Large (INRIA Futurs)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <sys/ptrace.h>
#include "common.h"
#include "sandbox.h"
#include <sys/time.h>
#include <sys/resource.h>

extern char **environ;
pid_t pid;

void redirect_sig(int sig) {
  kill(pid, sig);
}


int main(int argc, char *argv[])
{
  int i, exec_err, exit_code;
  sigset_t s;
  sigset_t olds;
  char path[MAX_PATH_SIZE];
  if(argc < 2)
    exit(1);

  //We save the curent directory
  getcwd(cwd, MAX_PATH_SIZE);
  //Initialize sandbox rules 
  generateSandboxRules();

  if(!which(argv[1], path)) {
    strncpy(path, argv[1], PATH_MAX);
  }

  if(!is_ptracable_bin(path)){
    fprintf(stderr, "%s: Error !\n\t %s is not a valid application\n", 
	    argv[0], argv[1]);
    exit(1);
  }

  switch (pid = fork()) {
  case -1:  // Fork error
    perror("fork");
    break;
  case 0: // The application fork
#ifdef VERBOSE
    fprintf(stderr, "Launch %s\n", path);
#endif
    //We nice down this thread to be sure the sandbox will allways prevail
    exec_err = getpriority(PRIO_PROCESS, getpid());
    nice(exec_err + 1);

    //Detatch this thread of the process group of the sandbox
    //And create a new process session
    setsid();

    //Launch the sandboxing
    ptrace(PTRACE_TRACEME, 0, 0, 0);
    //Execute the application
    exec_err = execvp(path, &argv[1]);

    //If execvp failed, we execute this part
    fprintf(stderr, "%s: %s: command not found\n", argv[0], argv[1]);
    exit(1);
    break;
  default:
    //Thread of the sandbox
    for(i = 1 ; i<_NSIG; i++)
      if(i != SIGTRAP && i!= SIGCHLD)
	signal(i, redirect_sig);
    exit_code = sandboxIt(argv[1]);

    //The end
#ifdef VERBOSE
    fprintf(stderr, "End of the sandboxing\n");
#endif //VERBOSE

    return exit_code;
  }
}
