/*
 * User Mode Sandbox for Grid
 * Copyright (C) 2004 Tangui Morlier
 * Copyright (C) 2004 Grand-Large (INRIA Futurs)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "ptrace_decode.h"

long decodeString(pid_t pid, long addr, char* chaine, int max_len){
  char *laddr;
  int i, j, size;
  union u {
    long val;
    char chars[sizeof(long)];
  }data;
  i = 0;
  laddr = chaine;
  while(1) {
    data.val = decodeWord(pid, addr + i * sizeof(long));
    if(data.val < 0)
      return -1;
    if(max_len < sizeof(long))
      data.chars[max_len]='\0';
    j = snprintf(laddr, max_len, data.chars);
    size += j;
    if(j<4) 
      break;
    ++i;
    laddr += sizeof(long);
    max_len -= sizeof(long);
  }
  return size;
}

long decodeWord(pid_t pid, long addr){
  return ptrace(PTRACE_PEEKDATA, pid, addr, NULL);
}

long decodeStruct(pid_t pid, long addr, void * struct_ptr, int size_struct){
  int i;
  union u {
    long val;
    char chars[sizeof(long)];
  }data;
  for(i = 0 ; i*sizeof(long) < size_struct ; i++) {
    data.val = decodeWord(pid, addr + i * sizeof(long));
    if(data.val < 0)
      return -1;
    memcpy(struct_ptr, &data.val, sizeof(long));
  }

  if(size_struct - i*sizeof(long) > 0){
    data.val = decodeWord(pid, addr + i * sizeof(long));
    if(data.val < 0)
      return -1;
    memcpy(struct_ptr, &data.val, size_struct - i*sizeof(long));
  }else if (size_struct - i*sizeof(long) < 0) 
    return -1;
  return size_struct;
}
