/*
 * User Mode Sandbox for Grid
 * Copyright (C) 2004 Tangui Morlier
 * Copyright (C) 2004 Grand-Large (INRIA Futurs)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef SANDBOXROLES_H
#define SANDBOXROLES_H
#include <sys/syscall.h>
#include "file_functions.h"
#include "fork_functions.h"
#include "common.h"


/**
 * generateSandboxRules() :
 *
 * Initialised the sandbox rules 
 *     This function determines if a syscall is allowed, denied or filtered 
 */
void generateSandboxRules();

int which(const char* file, char* buffer) ;

int is_ptracable_bin(const char * file);

/**
 * The syscall numbre to strings convertion
 */
extern char *(syscall_names[273]);

/**
 * rule_function :
 *
 *   This function definition determines if a call should be allowed or denied
 *
 * Returns:
 *     True if the syscall is allowed, false if not
 * Params:
 *     pi: sandbox info on the process the syscall was called
 *     ptr: Obsolete
 *     msg: If return False, this string explains why this syscall was denied
 *   MUST NOT EXECCED 255
 */
typedef int (*rule_function) (struct ptrace_info_struct * pi, char* ptr, char* msg);

/**
 * decode_function :
 *    
 *    This function definition decodes param values
 *    Those functions are called in a rule_function
 *
 * Params:
 *     pi: sandbox info on the process the syscall was called
 *     decode: the decoded structure 
 */
typedef void (*decode_function) (struct ptrace_info_struct * pi, void * decoded);


/**
 * syscall_rules :
 *
 *   An array of structure which dertermines for each syscall what to do
 *
 *   rules: array which pointes on two rule_function, one used on the 
 *   system call (rules[0]), the other the system call return (rules[1])
 *
 *   decoder: the parameter decoder if needed by the rules functions.
 *
 *   If the syscall_rules_struct is NULL, the syscall is allowed
 *
 *   The index of syscalls is determined in sys/syscall.h
 */
struct syscall_rules_struct {
  rule_function rules[2];
  decode_function decoder;
}syscall_rules[NB_SYSCALLS];

#endif//SANDBOXROLES_H
